@@ -35,6 +35,9 @@
 #'  \item{\code{getCapabilities()}}{
 #'    Get service capabilities. Inherited from OWS Client
 #'  }
+#'  \item{\code{reloadCapabilities()}}{
+#'    Reload service capabilities
+#'  }
 #'  \item{\code{describeFeatureType(typeName)}}{
 #'    Get the description of a given featureType
 #'  }
@@ -62,6 +65,11 @@ WFSClient <- R6Class("WFSClient",
        return(self$capabilities)
      },
      
+     #reloadCapabilities
+     reloadCapabilities = function(){
+       self$capabilities = WFSCapabilities$new(self$url, self$version, logger = self$loggerType)
+     },
+     
      #describeFeatureType
      describeFeatureType = function(typeName){
        self$INFO(sprintf("Fetching featureType description for '%s' ...", typeName))
